package com.bocloud.sms.booter.config;

import com.bocloud.sms.booter.websocket.logview.LogViewHandler;
import com.bocloud.sms.booter.websocket.message.MessageHandler;
import com.bocloud.sms.booter.websocket.message.MessageInterceptor;
import com.bocloud.sms.booter.websocket.upload.UploadServiceHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;
import org.springframework.web.socket.server.standard.ServerEndpointExporter;
import org.springframework.web.socket.server.standard.ServletServerContainerFactoryBean;
import org.springframework.web.socket.server.support.DefaultHandshakeHandler;

/**
 * 拦截器配置类
 *
 * @author tyl
 * @version 1.0
 * @since 2021年3月25日
 */
@Configuration
@EnableWebSocket
@RequiredArgsConstructor
public class WebSocketConfiguration implements WebSocketConfigurer {

    private final MessageInterceptor messageInterceptor;
    private final MessageHandler messageHandler;
    private final LogViewHandler logViewHandler;
    private final UploadServiceHandler uploadServiceHandler;

    @Bean
    public ServerEndpointExporter serverEndpointExporter() {
        return new ServerEndpointExporter();
    }

    @Bean
    public DefaultHandshakeHandler handshakeHandler() {
        return new DefaultHandshakeHandler();
    }

    @Bean
    public ServletServerContainerFactoryBean createWebSocketContainer() {
        ServletServerContainerFactoryBean container = new ServletServerContainerFactoryBean();
        container.setMaxTextMessageBufferSize(10240);
        container.setMaxBinaryMessageBufferSize(5 * 1024 * 1024);
        return container;
    }

    @Override
    public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
        // 注册普通消息服务
        registry.addHandler(messageHandler, "/messageService").addInterceptors(messageInterceptor)
                .setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*");
        registry.addHandler(messageHandler, "/sockjs/messageService").addInterceptors(messageInterceptor)
                .setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*").withSockJS();
        // 注册日志查看消息服务
        registry.addHandler(logViewHandler, "/logViewService").addInterceptors(messageInterceptor)
                .setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*");
        registry.addHandler(logViewHandler, "/sockjs/logViewService").addInterceptors(messageInterceptor)
                .setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*").withSockJS();
        // 普通文件上传消息服务
        registry.addHandler(this.uploadServiceHandler, "/uploadService").addInterceptors(messageInterceptor)
                .setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*");
        registry.addHandler(this.uploadServiceHandler, "/sockjs/uploadService")
                .addInterceptors(this.messageInterceptor).setHandshakeHandler(handshakeHandler()).setAllowedOrigins("*")
                .withSockJS();
    }
}
