package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.entity.Admission;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.repository.AdmissionRepository;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * IP访问控制
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/12
 */
@Component
@Slf4j
public class AdmissionUtil implements InitializingBean {


    private final AdmissionRepository admissionRepository;
    private final SystemConfigRepository systemConfigRepository;
    private final RedisTemplate<String, String> redisTemplate;

    public AdmissionUtil(AdmissionRepository admissionRepository, SystemConfigRepository systemConfigRepository, RedisTemplate<String, String> redisTemplate) {
        this.admissionRepository = admissionRepository;
        this.systemConfigRepository = systemConfigRepository;
        this.redisTemplate = redisTemplate;
    }

    public void refresh() {
        try {
            // 获取系统配置访问控制中IP访问策略
            SystemConfig strategy = systemConfigRepository.queryByCode("accessStrategy");
            if (strategy == null || !StringUtils.hasText(strategy.getValue())) {
                log.warn("Access strategy not acquired");
                return;
            }
            Admission.Type limitStrategy = Admission.Type.valueOf(strategy.getValue());
            List<Admission> admissionIps = admissionRepository.list();
            List<String> blacklist = new ArrayList<>();
            List<String> whitelist = new ArrayList<>();
            if (!ListTool.isEmpty(admissionIps)) {
                for (Admission admissionIp : admissionIps) {
                    if (Admission.Type.black == Admission.Type.valueOf(admissionIp.getType().name())
                            && Admission.Type.black == limitStrategy) {
                        blacklist.add(admissionIp.getIp());
                    } else if (Admission.Type.white == Admission.Type.valueOf(admissionIp.getType().name())
                            && Admission.Type.white == limitStrategy) {
                        whitelist.add(admissionIp.getIp());
                    }
                }
            }
            redisTemplate.opsForValue().set("blacklist", JSONArray.toJSONString(blacklist));
            redisTemplate.opsForValue().set("whitelist", JSONArray.toJSONString(whitelist));
        } catch (Exception e) {
            log.error("Get error message：{}", e.getMessage());
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.refresh();
    }

}
