package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSON;
import com.megatron.framework.registry.RegistryClient;
import com.megatron.framework.registry.RegistryProperties;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.RetryPolicy;
import org.apache.curator.framework.CuratorFramework;
import org.apache.curator.framework.CuratorFrameworkFactory;
import org.apache.curator.framework.api.ACLProvider;
import org.apache.curator.framework.recipes.cache.CuratorCache;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.cloud.bootstrap.BootstrapConfiguration;
import org.springframework.cloud.zookeeper.ZookeeperProperties;
import org.springframework.cloud.zookeeper.support.CuratorServiceDiscoveryAutoConfiguration;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Slf4j
@RequiredArgsConstructor
@Configuration
@BootstrapConfiguration
@AutoConfigureBefore({CuratorServiceDiscoveryAutoConfiguration.class})
@Component
public class ServiceWatcher implements InitializingBean {

    @Getter
    private Set<String> services;
    private final RegistryClient registryClient;
    private final ACLProvider aclProvider;
    private final RegistryProperties registryProperties;
    private final RetryPolicy retryPolicy;
    private final ZookeeperProperties properties;

    @Override
    public void afterPropertiesSet() throws Exception {
        CuratorFrameworkFactory.Builder builder = CuratorFrameworkFactory.builder();
        if (this.registryProperties.isSecurity() && StringUtils.hasText(this.registryProperties.getUsername()) &&
                StringUtils.hasText(this.registryProperties.getPassword())) {
            builder.authorization("digest", this.registryProperties.getUserPassword().getBytes());
        }
        builder.connectString(properties.getConnectString());

        builder.connectionTimeoutMs(this.registryProperties.getConnectTimeout());
        builder.sessionTimeoutMs(this.registryProperties.getSessionTimeout());
        builder.aclProvider(this.aclProvider);
        CuratorFramework curator = builder.retryPolicy(retryPolicy).build();
        curator.start();
        curator.blockUntilConnected(properties.getBlockUntilConnectedWait(), properties.getBlockUntilConnectedUnit());

        CuratorCache curatorCache = CuratorCache.build(curator, this.registryClient.getService().getPrefix());
        curatorCache.listenable().addListener((type, childData, childData1) -> {
            try {
                List<String> services = this.registryClient.getClient().getChildren(this.registryClient.getService().getPrefix());
                this.services = new HashSet<>(services);
                log.info("Refresh service data Success! services: " + JSON.toJSONString(services));
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw new RuntimeException(e);
            }
        });
        curatorCache.start();
    }

}
