package com.bocloud.sms.entity;

import com.bocloud.sms.model.MetaBean;
import com.bocloud.sms.model.ServiceBean;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 租户类
 *
 * @author weiwei
 * @version 1.0
 * @since 2016.08.04
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("tenant")
public class Tenant extends GenericEntity {
    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 邮箱
     */
    @Column("email")
    private String email;
    /**
     * 集团编码
     */
    @Column("group_code")
    private String groupCode;
    /**
     * 登录账号
     */
    @Column("account")
    private String account;
    /**
     * 密码
     */
    @JsonIgnore
    @Column("password")
    private String password;
    /**
     * 登录状态
     */
    @Column("login_status")
    private Boolean loginStatus;
    /**
     * 是否启用配额
     */
    @Column("quota_able")
    private Boolean quotaAble;
    /**
     * sessionId
     */
    @Column("session_id")
    private String sessionId;
    /**
     * 计费号
     */
    @Column("payment_code")
    private String paymentCode;
    /**
     * 联系人姓名
     */
    @Column("contact_name")
    private String contactName;
    /**
     * 头像
     */
    @Column("portrait")
    private String portrait;
    /**
     * 联系人邮箱
     */
    @Column("contact_email")
    private String contactEmail;
    /**
     * 联系人电话
     */
    @Column("contact_phone")
    private String contactPhone;
    /**
     * 账号到期时间
     */
    @Column("expired_date")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredDate;
    /**
     * 折扣率
     */
    @Column("discount")
    private BigDecimal discount;
    /**
     * 编码
     */
    @Column("code")
    private String code;
    // 优惠券数量
    @IgnoreAll
    private Integer coupons;
    // 授权的服务id
    @IgnoreAll
    private List<ServiceBean> services;
    // 最后一次密码修改时间
    @Column("last_pwd_modify_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastPwdModifyDate;
    // 最后一次登录时间
    @Column("last_login_date")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastLoginDate;

    /**
     * 租户授权的资源池平台类型
     */
    @IgnoreAll
    private Set<String> vendors;
    // 验证码
    @IgnoreAll
    private String captcha;
    // 运营平台订单数量
    @Column("cmp_orders")
    private Long cmpOrders;
    // esop订单数量
    @Column("esop_orders")
    private Long esopOrders;
    // 稽核状态
    @Column("audit_status")
    private String auditStatus;
    //最大快照数
    @Column("max_snapshot_count")
    private Integer maxSnapshotCount;

    /**
     * 组织id
     */
    @Column("depart_id")
    private Long departId;

    /**
     * 组织id
     */
    @Column("depart_ids")
    private String departIds;
    /**
     * 组织机构名称
     */
    @IgnoreAll
    private String departmentName;
    /**
     * 企业微信唯一标识userId
     */
    @Column("wechat")
    private String weChatWorkNumber;
    /**
     * 钉钉唯一标识userId
     */
    @Column("dingtalk")
    private String dingTalkNumber;

    @Ignore
    private List<Long> groupIds;

    @Ignore
    private List<MetaBean> metas;
}
