package com.bocloud.sms.repository;

import com.bocloud.sms.entity.SystemConfig;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author tyl
 * @time 2020/2/3 周一 18:24
 */
@Repository("systemConfigRepository")
public class SystemConfigRepository extends BasicGenericDao<SystemConfig, Long> {

    public SystemConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 系统设置修改
     *
     * @param systemConfig
     * @return
     * @throws Exception
     */
    public boolean modify(SystemConfig systemConfig) throws DataBaseException {
        return this.baseUpdate(systemConfig);
    }

    /**
     * 系统设置信息查询
     *
     * @param id
     * @return
     * @throws Exception
     */
    public SystemConfig query(Long id) throws DataBaseException {
        String sql = "select * from system_config where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<SystemConfig> systemConfigs = this.list(SystemConfig.class, sql, params);
        if (systemConfigs.isEmpty()) {
            return null;
        }
        return systemConfigs.get(0);
    }

    /**
     * 查询所有系统配置
     *
     * @return
     * @throws Exception
     */
    public List<SystemConfig> listAll() throws DataBaseException {
        String sql = "select a.* from system_config a";
        return this.list(SystemConfig.class, sql);
    }

    /**
     * 通过code查询信息
     *
     * @param code
     * @return
     * @throws Exception
     */
    public SystemConfig queryByCode(String code) throws DataBaseException {
        String sql = "select * from system_config where code = :code";
        Map<String, Object> params = MapTools.simpleMap(Common.CODE, code);
        List<SystemConfig> list = this.list(SystemConfig.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one system config in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 修改指定字段
     *
     * @param systemConfigs
     * @param fields
     * @return
     * @throws Exception
     */
    public boolean batchUpdate(List<SystemConfig> systemConfigs, String[] fields) throws DataBaseException {
        Integer batchSize = systemConfigs.size();
        return this.batchUpdate(systemConfigs, fields, batchSize);
    }

    public List<SystemConfig> listByCodes(Set<String> codes) throws DataBaseException {
        String sql = "select * from system_config where code in (:code)";
        Map<String, Object> params = MapTools.simpleMap(Common.CODE, codes);
        return this.list(SystemConfig.class, sql, params);
    }

    public List<SystemConfig> listByCategory(String category) throws DataBaseException {
        String sql = "select * from system_config where category = (:category)";
        Map<String, Object> params = MapTools.simpleMap("category", category);
        return this.list(SystemConfig.class, sql, params);
    }
}
